#include <string.h>
#include <stdlib.h>
#include <stdio.h>
  
#ifdef WIN32
  #include <windows.h>
  #include <winsock.h>
  #include <dos.h>
  #include <conio.h>
#else  
  #include <sys/types.h>
  #include <sys/socket.h>
  #include <netinet/in.h>
  #include <arpa/inet.h>
  #include <netdb.h>
  #include <unistd.h>
  #include <errno.h>

#endif
/*
 * UDP Connection Class.
 *
 */
class UDPConnection {
	
	private:
		
		/* General Configuration. */
		char *Server;
		int Port;
		int DataSize;
		
		/* Debug Mode ? */
		int debug;
		
		/* Socket Configuration. */
    #ifdef WIN32
    SOCKET sockfd;
    #else
		int sockfd;
    #endif
    		
		int state;
		
		
	public:
		char *buf;
		/* Store this here, so we can access it for Send(). */
		struct sockaddr_in server_addr;
		
		/*
		 * Configuration.
		 * Setup some basic configuration options & initialize class.
		 */
		UDPConnection(char *server, unsigned short port, unsigned int bufsiz) {
			Server = strdup(server);
			Port = port;
			DataSize = bufsiz;
			
			buf = (char*)malloc(bufsiz);
			state = 0;
			debug = 0;
      #ifdef WIN32
      sockfd = INVALID_SOCKET;
      #else
      sockfd = -1;
      #endif
      
      #ifdef WIN32
      WORD sockVersion;
      WSADATA wsaData;
      sockVersion = MAKEWORD(1, 1);
      WSAStartup(sockVersion, &wsaData);
      #endif
		}
		
		
		/*
		 * Connect()
		 * Connects to the server using the configuration information.
		 */
		int Connect() {
			struct hostent *host;
			struct sockaddr_in their_addr;
			char send_data[1024];
			
			#ifdef WIN32
      int nret;
      
      LPHOSTENT hostEntry = gethostbyname(Server);
      
      if (!hostEntry) {
        nret = WSAGetLastError();
        perror("gethostbyname");
        WSACleanup();
        return 0;
      }
      if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) == INVALID_SOCKET) {
        perror("socket");
        WSACleanup();
        return(0);
      }
      
      
      SOCKADDR_IN server_addr;
      server_addr.sin_family = AF_INET;
      server_addr.sin_addr = *((LPIN_ADDR)*hostEntry->h_addr_list);
      server_addr.sin_port = htons(Port);
      #else
			
			host= (struct hostent *) gethostbyname((char *)Server);
			
			if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
				perror("socket");
				return(0);
			}
			
			server_addr.sin_family	= AF_INET;
			server_addr.sin_port		= htons(Port);
			server_addr.sin_addr		= *((struct in_addr *)host->h_addr);
			bzero(&(server_addr.sin_zero),8);
      #endif
      
			return(1);
		 }
		 
		/*
		 * Send( String , Size of String )
		 * Sends a string to the server.
		 */
		int Send(char *outbuf, int size) {
			sendto(sockfd, outbuf, size, 0, (struct sockaddr *)&server_addr, sizeof(struct sockaddr));
		}

};
